@@ -331,7 +331,10 @@ class ImageCache:
 
     def __insert_file(self, file):
         file_insert = "INSERT OR REPLACE INTO file(folder_id, basename, extension, last_modified) VALUES((SELECT folder_id from folder where name = ?), ?, ?, ?)"
-        folder_insert = "INSERT OR REPLACE INTO folder(name, missing) VALUES(?, 0)"
+        # Insert the new folder if it's not already in the table. Update the missing field separately.
+        folder_insert = "INSERT OR IGNORE INTO folder(name) VALUES(?)"
+        folder_update = "UPDATE folder SET missing = 0 where name = ?"
+
         mod_tm =  os.path.getmtime(file)
         dir, file_only = os.path.split(file)
         base, extension = os.path.splitext(file_only)
@@ -344,6 +347,7 @@ class ImageCache:
 
         # Insert this file's info into the folder, file, and meta tables
         self.__db.execute(folder_insert, (dir,))
+        self.__db.execute(folder_update, (dir,))
         self.__db.execute(file_insert, (dir, base, extension.lstrip("."), mod_tm))
         self.__db.execute(meta_insert, vals)
 
